import { useEffect, useState } from 'react';
import { useSelector, useDispatch } from 'react-redux'
import AppBar from './AppBar';
import RecordList from './RecordList';
import PlayerBar from './PlayerBar';
import store from './business/store';
import yzs from "./business/request.js";
import { setList, setCurrentLyric, setCurrentBlob, setCurrentWaveData } from "./business/recorderSlice.js"
import { CssBaseline, Box } from '@mui/material';
import RecordLyrics from './RecordLyrics';
import { createTheme, ThemeProvider, styled } from '@mui/material/styles';
import expand from './assets/expand.png';
import close from './assets/close.png';

const drawerWidth = 240;

const theme = createTheme({
    status: {
        danger: '#e53e3e',
    },
    palette: {
        primary: {
            main: '#FF595A',
            darker: '#FF595A',
        },
        neutral: {
            main: '#64748B',
            contrastText: '#fff',
        },
        black: {
            main: "#222222",
        }
    },
});


function fetchRecord(accessToken, record) {
    yzs.download(accessToken, record.transResultUrl).then(
        blob => blob.text()
    ).then(text => {
        // console.log("type", record.type, text);
        let payload = record.type === 1 ? JSON.parse(text) : text;
        store.dispatch(setCurrentLyric(payload));
    });
    yzs.download(accessToken, record.audioUrl).then(blob => {
        store.dispatch(setCurrentBlob(URL.createObjectURL(blob)));
        blob.arrayBuffer().then(arrayBuffer => {
            let context = new (window.AudioContext || window.webkitAudioContext)();
            context.decodeAudioData(arrayBuffer).then(audioBuffer => {
                let interval = audioBuffer.sampleRate / 1000 * 100;
                let waveData = audioBuffer.getChannelData(0);
                let wave = [];
                let amplitude = 0;
                for (let i = 0; i < waveData.length; i++) {
                    amplitude += Math.abs(waveData[i]);
                    if (i % interval == 0) {
                        wave.push(Math.floor(amplitude * 500 / interval));
                        amplitude = 0;
                    }
                }
                store.dispatch(setCurrentWaveData(wave));
            })
        });
    });
}

const ClickHanlde = styled('div', { shouldForwardProp: (prop) => prop !== 'open' })(
    ({ theme, open }) => ({
        width: 18,
        height: 50,
        backgroundImage: `url(${open ? close : expand})`,
        position: "fixed",
        top: "50%",
        left: 0,
        zIndex: theme.zIndex.drawer + 1,
        transition: theme.transitions.create('left', {
            easing: theme.transitions.easing.sharp,
            duration: theme.transitions.duration.leavingScreen,
        }),
        ...(open && {
            transition: theme.transitions.create('left', {
                easing: theme.transitions.easing.easeOut,
                duration: theme.transitions.duration.enteringScreen,
            }),
            left: drawerWidth - 18,
        }),
    })
);


const Main = styled('main', { shouldForwardProp: (prop) => prop !== 'open' })(
    ({ theme, open }) => ({
        flex: 1,
        padding: theme.spacing(3),
        transition: theme.transitions.create('margin', {
            easing: theme.transitions.easing.sharp,
            duration: theme.transitions.duration.leavingScreen,
        }),
        marginLeft: `-${drawerWidth}px`,
        ...(open && {
            transition: theme.transitions.create('margin', {
                easing: theme.transitions.easing.easeOut,
                duration: theme.transitions.duration.enteringScreen,
            }),
            marginLeft: 0,
        }),
    }),
);

export default function () {
    const dispatch = useDispatch()
    const accessToken = useSelector(state => state.user.accessToken);
    const passportId = useSelector(state => state.user.passportId);
    const currentTime = useSelector(state => state.recorder.currentTime);
    const currentLyric = useSelector(state => state.recorder.currentLyric);
    const [playerBarWidth, setPlayerBarWidth] = useState(0);
    const [open, setOpen] = useState(true);
    useEffect(() => {
        yzs.get_record_list(accessToken, passportId).then(list => {
            dispatch(setList(list.result));
            if (list.result.length > 0) {
                fetchRecord(accessToken, list.result.at(0));
            }
        }).catch(error => {
            console.log("get list failed", error);
        });
    }, [accessToken, passportId]);

    const onClick = () => {
        setOpen(!open);
        setPlayerBarWidth(document.documentElement.clientWidth - 240 - 48); // 防止中途底部出现scrollbar
    }

    const handleResize = () => {
        console.log("innerWidth", document.documentElement.clientWidth, document.documentElement.clientWidth - (open ? 240 : 0) - 48)
        setPlayerBarWidth(document.documentElement.clientWidth - (open ? 240 : 0) - 48);
    }

    const onTransitionEnd = () => {
        handleResize();
    }

    useEffect(() => {
        handleResize();
        window.addEventListener('resize', handleResize);
        return () => {
            window.removeEventListener('resize', handleResize);
        };
    }, []);

    useEffect(() => { handleResize(); }, [currentLyric]);

    return <Box sx={{ display: 'flex' }}>
        <ThemeProvider theme={theme}>
            <CssBaseline />
            <AppBar />
            <RecordList open={open} fetchRecord={fetchRecord} />
            <ClickHanlde open={open} onClick={onClick} />
            <Main open={open}
                onTransitionEnd={onTransitionEnd}
            >
                <PlayerBar width={playerBarWidth} currentTime={currentTime} />
                <RecordLyrics currentLyric={currentLyric} currentTime={currentTime} />
            </Main>
        </ThemeProvider>
    </Box >
}